Content.makeFrontInterface(1100, 700);

const var Arpeggiator1 = Synth.getMidiProcessor("Arpeggiator1");

// ----------------------- Slider Packs ---------------------------------------
const var SliderPack1 = Content.getComponent("SliderPack1");


// -------------------------- Buttons -----------------------------------------
const var ArpTieOnOff = Content.getComponent("ArpTieOnOff");
const var SortKeys = Content.getComponent("SortKeys");

// -------------------------- ComboBox --------------------------------------- 
const var DirectionCtrl = Content.getComponent("DirectionCtrl");

// --------------------------- Knobs ------------------------------------------
const var StepCtrl2 = Content.getComponent("StepCtrl2");
const var StepReset2 = Content.getComponent("StepReset2");
const var StrideCtrl2 = Content.getComponent("StrideCtrl2");
const var SpeedCtrl2 = Content.getComponent("SpeedCtrl2");
const var OctCtrl2 = Content.getComponent("OctCtrl2");
const var ShuffleCtrl2 = Content.getComponent("ShuffleCtrl2");

// ----------------------------------------------------------------------------
const var browserPanel = Content.getComponent("browserPanel");

const var SavePreset = Content.getComponent("SavePreset");
const var SavePresetOptions = Content.getComponent("SavePresetOptions");
const var PresetNameToSave = Content.getComponent("PresetNameToSave");

inline function onpresetBrowserControl(component, value)
{
	browserPanel.showControl(value);
	PresetBrowser.populatePresetList();
	SavePresetOptions.showControl(0);
    SavePreset.setValue(0);
    PresetNameToSave.set("text", "choose name...");

};

Content.getComponent("presetBrowser").setControlCallback(onpresetBrowserControl);

// Defines all controls and callbacks for preset browser
namespace PresetBrowser
{
    const var PresetList = Content.getComponent("PresetList");
    PresetList.setControlCallback(onPresetListControl);
    
    const var ShowPresetFolder = Content.getComponent("ShowPresetFolder");
    ShowPresetFolder.setControlCallback(onShowPresetFolderControl);
    
    const var SavePreset = Content.getComponent("SavePreset");
    SavePreset.setControlCallback(onSavePresetControl);
    
    const var SavePresetOptions = Content.getComponent("SavePresetOptions");
    const var presetSavePanel = Content.getComponent("presetSavePanel");
    const var PresetNameToSave = Content.getComponent("PresetNameToSave");
    
    const var SaveOk = Content.getComponent("SaveOk");
    SaveOk.setControlCallback(onSaveOkControl);

    const var SaveCancel = Content.getComponent("SaveCancel");
    SaveCancel.setControlCallback(onSaveCancelControl);
    
    populatePresetList();
    
    // Populates preset viewport by fetching all file names in the UserPreset directory
    inline function populatePresetList()
    {
        local presetNames = PresetHandler.fetchPresetNames();
        PresetList.set("items", presetNames.join("\n"));
    }
    
// -----------------------------------------------------------------------------------
    inline function onPresetListControl(component, value)
    {
        local items = component.get("items").split("\n");
        local preset = PresetHandler.loadPresetFile(items[value]);
        setControlValues(preset);
    };

// ----------------------------------------------------------------------------------
    inline function onShowPresetFolderControl(component, value)
    {
        if (value) PresetHandler.showPresetFolder();
    };
    
// ---------------------------------------------------------------------------------    
reg SliderPack1SAVEData = [];
    
// ---------------------------------------------------------------------------------
    inline function onSavePresetControl(component, value)
    {
        SavePresetOptions.showControl(value);     
    };

// ---------------------------------------------------------------------------------
    inline function onSaveOkControl(component, value) 
    {
        if (value)
        {
            local presetName = PresetNameToSave.get("text");
            PresetHandler.savePresetFile(presetName, getControlValues());
            SavePreset.setValue(false);
            SavePreset.changed();
            populatePresetList();
        }
    };

// ---------------------------------------------------------------------------------
    inline function onSaveCancelControl(component, value)
    {
        SavePreset.setValue(false);
        SavePreset.changed();
    };

// ---------------------------------------------------------------------------------    
const SliderPack1Data = {};
    
    inline function getControlValues() 
    {
              for (i = 0; i < 128; i++)
        {
            SliderPack1SAVEData[i] = SliderPack1.getSliderValueAt(i);
                      
        }   
        
        return 
        
        {
            
        // Knob Save Values            
            "StepCtrl2Data" : StepCtrl2.getValue(),
            "StepReset2Data" : StepReset2.getValue(),
            "StrideCtrl2Data" : StrideCtrl2.getValue(),
            "SpeedCtrl2Data" : SpeedCtrl2.getValue(),
            "OctCtrl2Data" : OctCtrl2.getValue(),
            "ShuffleCtrl2Data" : ShuffleCtrl2.getValue(),
            
        // Button Save Values  
            "ArpTieOnOffData" : ArpTieOnOff.getValue(),
            "SortKeysData" : SortKeys.getValue(),
            
        // ComboBox Save Values    
            "DirectionCtrlData" : DirectionCtrl.getValue(),       

         // SAVE SLIDERPACK VALUES
            "SliderPack1Data" : SliderPack1SAVEData
                 
        };
    }
    
// ------------------------------------------------------------------------------------- 
    inline function setControlValues(data)
    {
        // Load Buttons Saved Values Into Buttons
        ArpTieOnOff.setValue(data.ArpTieOnOffData);
        ArpTieOnOff.changed();      
   
        SortKeys.setValue(data.SortKeysData);
        SortKeys.changed();
        
        // Load Knobs Saved Values Into Knobs        
        StepCtrl2.setValue(data.StepCtrl2Data);
        StepCtrl2.changed();
        
        StepReset2.setValue(data.StepReset2Data);
        StepReset2.changed();
        
        StrideCtrl2.setValue(data.StrideCtrl2Data);
        StrideCtrl2.changed();
        
        SpeedCtrl2.setValue(data.SpeedCtrl2Data);
        SpeedCtrl2.changed();
        
        OctCtrl2.setValue(data.OctCtrl2Data);
        OctCtrl2.changed();
        
        ShuffleCtrl2.setValue(data.ShuffleCtrl2Data);
        ShuffleCtrl2.changed();
        
         // Load ConboBox Saved Values Into ConboBox        
        DirectionCtrl.setValue(data.DirectionCtrlData);
        DirectionCtrl.changed();        
        
        // LOAD SAVED SLIDERPACK VALUES INTO SLIDERPACKS
        for (i = 0; i < data.SliderPack1Data.length; i++)
        {
            SliderPack1.setSliderAtIndex(i, data.SliderPack1Data[i]);
     
        }  
                   
    }
}

// Provides functions for loading and saving preset files from the file system
namespace PresetHandler
{
    inline function getPresetFolder() {
        return FileSystem.getFolder(FileSystem.UserPresets).getChildFile("ARP");
    }
    
	inline function fetchPresetNames() {
		local presets = FileSystem.findFiles(getPresetFolder(), "*.arp", true);
		local presetNames = [];
		for (file in presets)
			presetNames.push(file.toString(1));			
		return presetNames;
	}

	inline function loadPresetFile(presetName) {
		local presetFile = FileSystem.findFiles(getPresetFolder(), presetName + ".arp", true);
		return presetFile[0].loadAsObject();
	}

	inline function savePresetFile(presetName, data) {
		local presetFile = getPresetFolder().getChildFile(presetName + ".arp");
		presetFile.writeObject(data);
	}

	inline function presetNameExists(presetName) {
		return getPresetFolder().getChildFile(presetName + ".arp").isFile();
	}

	inline function showPresetFolder() {
		getPresetFolder().show();
	}
}function onNoteOn()
{
	
}
 function onNoteOff()
{
	
}
 function onController()
{
	
}
 function onTimer()
{
	
}
 function onControl(number, value)
{
	
}
 